from email import message_from_bytes
from email import policy
from email.header import decode_header
from imaplib import IMAP4_SSL
from typing import List, Dict, Union

from je_mail_thunder.utils.exception.exception_tags import mail_thunder_content_login_failed
from je_mail_thunder.utils.logging.loggin_instance import mail_thunder_logger
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import read_output_content
from je_mail_thunder.utils.save_mail_user_content.save_on_env import get_mail_thunder_os_environ


class IMAPWrapper(IMAP4_SSL):

    def __init__(self, host: str = 'imap.gmail.com'):
        super().__init__(host)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
        self.logout()

    def imap_later_init(self):
        """
        Try to log in
        :return: None
        """
        mail_thunder_logger.info(f"MT_imap_later_init")
        try:
            self.imap_try_to_login_with_env_or_content()
        except Exception as error:
            mail_thunder_logger.error(f"imap_later_init, failed: {repr(error)}")

    def imap_try_to_login_with_env_or_content(self):
        """
        Try to find user and password on cwd /mail_thunder_content.json or env var
        :return: None
        """
        mail_thunder_logger.info(f"imap_try_to_login_with_env_or_content")
        try:
            user_info = read_output_content()
            if user_info is not None and type(user_info) == dict:
                if user_info.get("user", None) is not None and user_info.get("password", None) is not None:
                    self.login(user_info.get("user"), user_info.get("password"))
            else:
                user_info = get_mail_thunder_os_environ()
                if user_info is not None and type(user_info) == dict:
                    if user_info.get("mail_thunder_user", None) is not None and user_info.get(
                            "mail_thunder_user_password", None) is not None:
                        self.login(user_info.get("mail_thunder_user"), user_info.get("mail_thunder_user_password"))
        except Exception as error:
            mail_thunder_logger.info(
                f"imap_try_to_login_with_env_or_content, "
                f"failed: {repr(error) + ' ' + mail_thunder_content_login_failed}")

    def imap_select_mailbox(self, mailbox: str = "INBOX", readonly: bool = False):
        """
        :param mailbox: Mailbox we want to select like INBOX
        :param readonly: Readonly or not
        :return: None
        """
        mail_thunder_logger.info(f"imap_select_mailbox, mailbox: {mailbox}, readonly: {readonly}")
        try:
            select_status = self.select(mailbox=mailbox, readonly=readonly)
            return True if select_status[0] == "OK" else False
        except Exception as error:
            mail_thunder_logger.error(
                f"imap_select_mailbox, mailbox: {mailbox}, readonly: {readonly}, failed: {repr(error)}")

    def imap_search_mailbox(self, search_str: [str, list] = "ALL", charset: str = None) -> list:
        """
        Get all mail detail as list
        :param search_str: Search pattern
        :param charset: Charset pattern
        :return: All mail detail as list [mail_response, mail_decode, mail_content]
        """
        mail_thunder_logger.info(f"imap_search_mailbox, search_str: {search_str}, charset: {charset}")
        try:
            response, mail_number_string = self.search(charset, search_str)
            mail_detail_list = list()
            for num_of_mail in mail_number_string[0].split():
                response, mail_data = self.fetch(num_of_mail, "(RFC822)")
                mail_data: List[List]
                # [0][1] is message data [0][0] is message decode like RFC822 {565}
                message = message_from_bytes(mail_data[0][1], policy=policy.default)
                mail_detail_list.append([response, mail_data[0][0], message])
            return mail_detail_list
        except Exception as error:
            mail_thunder_logger.error(
                f"imap_search_mailbox, search_str: {search_str}, charset: {charset}, failed: {repr(error)}")

    def imap_mail_content_list(
            self, search_str: [str, list] = "ALL", charset: str = None) -> List[Dict[str, Union[str, bytes]]]:
        mail_thunder_logger.info(f"imap_mail_content_list, search_str: {search_str}, charset: {charset}")
        """
        Get all mail content as list
        :param search_str: Search pattern
        :param charset: Charset pattern 
        :return: All mail content as list [{"SUBJECT": "mail_subject", "FROM": "mail_from", "TO": "mail_to"}]
        """
        try:
            mail_list = self.imap_search_mailbox(search_str, charset)
            mail_content_dict = dict()
            mail_content_list = list()
            for mail_data in mail_list:
                mail = mail_data[2]
                mail_content_dict.update({"SUBJECT": mail.get("Subject")})
                mail_content_dict.update({"FROM": mail.get("FROM")})
                mail_content_dict.update({"TO": mail.get("TO")})
                body = ""
                if mail.is_multipart():
                    for part in mail.get_payload():
                        body = part.get_payload(decode=False)
                else:
                    body = mail.get_payload(decode=False)
                body = str(decode_header(str(body))[0][0])
                mail_content_dict.update({"BODY": body})
                mail_content_list.append(mail_content_dict)
                mail_content_dict = dict()
            return mail_content_list
        except Exception as error:
            mail_thunder_logger.error(
                f"imap_mail_content_list, search_str: {search_str}, charset: {charset}, failed: {repr(error)}")

    def imap_output_all_mail_as_file(
            self, search_str: [str, list] = "ALL", charset: str = None) -> List[Dict[str, Union[str, bytes]]]:
        mail_thunder_logger.info(f"imap_mail_content_list, search_str: {search_str}, charset: {charset}")
        """
        Get all mail content data and output as file
        :param search_str: Search pattern
        :param charset: Charset pattern 
        :return: All mail content as list [{"SUBJECT": "mail_subject", "FROM": "mail_from", "TO": "mail_to"}]
        """
        try:
            all_mail = self.imap_mail_content_list(search_str=search_str, charset=charset)
            same_name_dict: Dict[str, int] = dict()
            for mail in all_mail:
                if same_name_dict.get((mail.get("SUBJECT"))) is None:
                    same_name_dict.update({mail.get("SUBJECT"): 0})
                else:
                    same_name_dict.update({mail.get("SUBJECT"): same_name_dict.get(mail.get("SUBJECT")) + 1})
                with open(mail.get("SUBJECT") + str(same_name_dict.get(mail.get("SUBJECT"))), "w+") as file:
                    if isinstance(mail.get("BODY"), bytes):
                        file.write(mail.get("BODY").decode("utf-8"))
                    else:
                        file.write(mail.get("BODY"))
            return all_mail
        except Exception as error:
            mail_thunder_logger.error(
                f"imap_mail_content_list, search_str: {search_str}, charset: {charset}, failed: {repr(error)}")

    def imap_quit(self):
        """
        Quit service and close connect
        :return: None
        """
        mail_thunder_logger.info(f"MT_imap_quit")
        try:
            self.close()
            self.logout()
        except Exception as error:
            mail_thunder_logger.error(f"imap_quit, failed: {repr(error)}")


imap_instance = IMAPWrapper()



# Json exception
mail_thunder_cant_reformat_json_error = "Can't reformat json is type right?"
mail_thunder_wrong_json_data_error = "Can't parser json"
# Content data exception
mail_thunder_content_set_compiler_error = "When set compiler using content make an error"
mail_thunder_content_file_error = "MailThunder content file error"
mail_thunder_content_login_failed = "can't login with mail thunder content"
# Argparse
mail_thunder_service_file_error = "service param -s got the wrong data"
mail_thunder_login_error = "need set --user and --password to login"
mail_thunder_argparse_get_wrong_function = "get unknown function"
# Add command
add_command_exception: str = "command value type should be as method or function"
# Executor
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# Json file exception
cant_execute_action_error: str = "cant execute action"
cant_generate_json_report: str = "can't generate json report"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
action_is_null_error: str = "json action is null"



import builtins
import types
from inspect import getmembers, isbuiltin
from typing import Union, Any

from je_mail_thunder.imap.imap_wrapper import imap_instance
from je_mail_thunder.smtp.smtp_wrapper import smtp_instance
from je_mail_thunder.utils.exception.exception_tags import cant_execute_action_error, executor_list_error, \
    action_is_null_error, add_command_exception
from je_mail_thunder.utils.exception.exceptions import ExecuteActionException, AddCommandException
from je_mail_thunder.utils.json.json_file import read_action_json
from je_mail_thunder.utils.logging.loggin_instance import mail_thunder_logger
from je_mail_thunder.utils.package_manager.package_manager_class import package_manager
from je_mail_thunder.utils.save_mail_user_content.save_on_env import set_mail_thunder_os_environ, \
    get_mail_thunder_os_environ
from je_mail_thunder.utils.scheduler.extend_apscheduler import scheduler_manager


class Executor(object):

    def __init__(self):
        self.event_dict: dict = {
            # SMTP
            "MT_smtp_later_init": smtp_instance.smtp_later_init,
            "MT_smtp_create_message_with_attach_and_send": smtp_instance.smtp_create_message_with_attach_and_send,
            "MT_smtp_create_message_and_send": smtp_instance.smtp_create_message_and_send,
            "smtp_quit": smtp_instance.quit,
            # IMAP
            "MT_imap_later_init": imap_instance.imap_later_init,
            "MT_imap_select_mailbox": imap_instance.imap_select_mailbox,
            "MT_imap_search_mailbox": imap_instance.imap_search_mailbox,
            "MT_imap_mail_content_list": imap_instance.imap_mail_content_list,
            "MT_imap_output_all_mail_as_file": imap_instance.imap_output_all_mail_as_file,
            "MT_imap_quit": imap_instance.imap_quit,
            # Content
            "MT_set_mail_thunder_os_environ": set_mail_thunder_os_environ,
            "MT_get_mail_thunder_os_environ": get_mail_thunder_os_environ,
            # Package Manager
            "MT_add_package_to_executor": package_manager.add_package_to_executor,
            # Scheduler
            "MT_scheduler_event_trigger": self.scheduler_event_trigger,
            "MT_remove_blocking_scheduler_job": scheduler_manager.remove_blocking_job,
            "MT_remove_nonblocking_scheduler_job": scheduler_manager.remove_nonblocking_job,
            "MT_start_blocking_scheduler": scheduler_manager.start_block_scheduler,
            "MT_start_nonblocking_scheduler": scheduler_manager.start_nonblocking_scheduler,
            "MT_start_all_scheduler": scheduler_manager.start_all_scheduler,
            "MT_shutdown_blocking_scheduler": scheduler_manager.shutdown_blocking_scheduler,
            "MT_shutdown_nonblocking_scheduler": scheduler_manager.shutdown_nonblocking_scheduler,
        }
        # get all builtin function and add to event dict
        for function in getmembers(builtins, isbuiltin):
            self.event_dict.update({str(function[0]): function[1]})

    def _execute_event(self, action: list):
        event = self.event_dict.get(action[0])
        if len(action) == 2:
            if isinstance(action[1], dict):
                return event(**action[1])
            else:
                return event(*action[1])
        elif len(action) == 1:
            return event()
        else:
            raise ExecuteActionException(cant_execute_action_error + " " + str(action))

    def execute_action(self, action_list: [list, dict]) -> dict:
        """
        use to execute all action on action list(action file or program list)
        :param action_list the list include action
        for loop the list and execute action
        """
        if isinstance(action_list, dict):
            action_list: list = action_list.get("auto_control", None)
            if action_list is None:
                raise ExecuteActionException(executor_list_error)
        execute_record_dict = dict()
        try:
            if len(action_list) > 0 or isinstance(action_list, list):
                pass
            else:
                raise ExecuteActionException(action_is_null_error)
        except Exception as error:
            mail_thunder_logger.error(
                f"Execute {action_list} failed. {repr(error)}"
            )
        for action in action_list:
            try:
                event_response = self._execute_event(action)
                execute_record = "execute: " + str(action)
                mail_thunder_logger.info(
                    f"Execute {action}"
                )
                execute_record_dict.update({execute_record: event_response})
            except Exception as error:
                mail_thunder_logger.error(
                    f"Execute {action} failed. {repr(error)}"
                )
                execute_record = "execute: " + str(action)
                execute_record_dict.update({execute_record: repr(error)})
        for key, value in execute_record_dict.items():
            print(key, flush=True)
            print(value, flush=True)
        return execute_record_dict

    def execute_files(self, execute_files_list: list) -> list:
        """
        :param execute_files_list: list include execute files path
        :return: every execute detail as list
        """
        execute_detail_list: list = list()
        for file in execute_files_list:
            execute_detail_list.append(self.execute_action(read_action_json(file)))
        return execute_detail_list

    def scheduler_event_trigger(
            self, function: str, id: str = None, args: Union[list, tuple] = None,
            kwargs: dict = None, scheduler_type: str = "nonblocking", wait_type: str = "secondly",
            wait_value: int = 1, **trigger_args: Any) -> None:
        if scheduler_type == "nonblocking":
            scheduler_event = scheduler_manager.nonblocking_scheduler_event_dict.get(wait_type)
        else:
            scheduler_event = scheduler_manager.blocking_scheduler_event_dict.get(wait_type)
        scheduler_event(self.event_dict.get(function), id, args, kwargs, wait_value, **trigger_args)


executor = Executor()
package_manager.executor = executor


def add_command_to_executor(command_dict: dict):
    """
    :param command_dict: dict include command we want to add to event_dict
    """
    mail_thunder_logger.info(
        f"Add command to executor {command_dict}"
    )
    for command_name, command in command_dict.items():
        if isinstance(command, (types.MethodType, types.FunctionType)):
            executor.event_dict.update({command_name: command})
        else:
            raise AddCommandException(add_command_exception)


def execute_action(action_list: list) -> dict:
    return executor.execute_action(action_list)


def execute_files(execute_files_list: list) -> list:
    return executor.execute_files(execute_files_list)



template_keyword_1: list = [
    ["MT_smtp_later_init"],
    ["MT_smtp_create_message_and_send", {
        "message_content": "test", "message_setting_dict": {
            "Subject": "test_subject",
            "To": "example@gmail.com",
            "From": "example@gmail.com"
        }}
     ],
    ["smtp_quit"]
]

template_keyword_2: list = [
    ["MT_imap_later_init"],
    ["MT_imap_select_mailbox"],
    ["MT_imap_output_all_mail_as_file"]
]

bad_template_1 = [
    ["MT_add_package_to_executor", ["os"]],
    ["os_system", ["python --version"]],
    ["os_system", ["python -m pip --version"]],
]



import smtplib
from email.message import EmailMessage
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from mimetypes import guess_type
from os import path
from smtplib import SMTP_SSL

from je_mail_thunder.utils.logging.loggin_instance import mail_thunder_logger
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import read_output_content
from je_mail_thunder.utils.save_mail_user_content.save_on_env import get_mail_thunder_os_environ


class SMTPWrapper(SMTP_SSL):

    def __init__(self, host: str = "smtp.gmail.com", port: int = 465):
        super().__init__(host, port)
        self.login_state = False

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.quit()

    def smtp_later_init(self):
        """
        Try to log in
        :return: None
        """
        mail_thunder_logger.info("MT_smtp_later_init")
        try:
            self.smtp_try_to_login_with_env_or_content()
        except Exception as error:
            mail_thunder_logger.error(f"smtp_later_init, failed: {error}")

    @staticmethod
    def smtp_create_message(message_content: str, message_setting_dict: dict, **kwargs):
        """
        Create new EmailMessage instance
        :param message_content: Mail content
        :param message_setting_dict: Dict include SUBJECT FROM TO and another EmailMessage Key and Value
        :param kwargs: EmailMessage setting
        :return: None
        """
        mail_thunder_logger.info(
            f"smtp_create_message, message_content{message_content}, message_setting_dict: {message_setting_dict}")
        try:
            message = EmailMessage(**kwargs)
            message.set_content(message_content)
            for key, value in message_setting_dict.items():
                message[key] = value
            return message
        except Exception as error:
            mail_thunder_logger.error(
                f"smtp_create_message, message_content{message_content}, "
                f"message_setting_dict: {message_setting_dict}, failed: {repr(error)}")

    @staticmethod
    def smtp_create_message_with_attach(message_content: str, message_setting_dict: dict,
                                        attach_file: str, use_html: bool = False):
        """
        Create new EmailMessage with attach file instance
        :param message_content: Mail content
        :param message_setting_dict: Dict include SUBJECT FROM TO and another EmailMessage Key and Value
        :param attach_file: File path as str
        :param use_html: Enable HTML format (If attach file is html)
        :return: None
        """
        mail_thunder_logger.info(
            f"smtp_create_message_with_attach, message_content{message_content}, "
            f"message_setting_dict: {message_setting_dict}, attach_file: {attach_file}, use_html: {use_html}")
        try:
            message = MIMEMultipart()
            for key, value in message_setting_dict.items():
                message[key] = value
            if use_html:
                mime_part = MIMEText(message_content, "html")
            else:
                mime_part = MIMEText(message_content)
            message.attach(mime_part)
            content_type, encoding = guess_type(attach_file)
            if content_type is None or encoding is not None:
                content_type = "application/octet-stream"
            main_type, sub_type = content_type.split("/", 1)
            if main_type == "text":
                file_read = open(attach_file, "r+")
                mime_part = MIMEText(file_read.read(), _subtype=sub_type)
                file_read.close()
            elif main_type == "image":
                file_read = open(attach_file, "rb")
                mime_part = MIMEImage(file_read.read(), _subtype=sub_type)
                file_read.close()
            elif main_type == "audio":
                file_read = open(attach_file, "rb")
                mime_part = MIMEAudio(file_read.read(), _subtype=sub_type)
                file_read.close()
            else:
                file_read = open(attach_file, "rb")
                mime_part = MIMEBase(main_type, sub_type)
                mime_part.set_payload(file_read.read())
                file_read.close()
            filename = path.basename(attach_file)
            mime_part.add_header("Content-Disposition", "attachment", filename=filename)
            mime_part.add_header("Content-ID", "{filename}".format(filename=filename))
            message.attach(mime_part)
            return message
        except Exception as error:
            mail_thunder_logger.info(
                f"smtp_create_message_with_attach, message_content{message_content}, "
                f"message_setting_dict: {message_setting_dict}, attach_file: {attach_file}, "
                f"use_html: {use_html}, failed: {repr(error)}")

    def smtp_try_to_login_with_env_or_content(self):
        """
        Try to find user and password on cwd /mail_thunder_content.json or env var
        :return: None
        """
        mail_thunder_logger.info(f"smtp_try_to_login_with_env_or_content")
        try:
            user_info = read_output_content()
            self.login_state = False
            try:
                if user_info is not None and type(user_info) == dict:
                    if user_info.get("user", None) is not None and user_info.get("password", None) is not None:
                        self.login(user_info.get("user"), user_info.get("password"))
                        self.login_state = True
                else:
                    user_info = get_mail_thunder_os_environ()
                    if user_info is not None and type(user_info) == dict:
                        if user_info.get("mail_thunder_user", None) is not None and user_info.get(
                                "mail_thunder_user_password", None) is not None:
                            self.login(user_info.get("mail_thunder_user"), user_info.get("mail_thunder_user_password"))
                            self.login_state = True
                return self.login_state
            except smtplib.SMTPAuthenticationError as error:
                mail_thunder_logger.info(f"smtp_try_to_login_with_env_or_content, failed: {repr(error)}")
                return self.login_state
        except Exception as error:
            mail_thunder_logger.info(f"smtp_try_to_login_with_env_or_content, failed: {repr(error)}")

    def quit(self):
        """
        Quit service and close connect
        :return: None
        """
        mail_thunder_logger.info(f"SMTP quit")
        self.login_state = False

    def smtp_create_message_with_attach_and_send(self, message_content: str, message_setting_dict: dict,
                                                 attach_file: str, use_html: bool = False):
        """
        Create new EmailMessage with attach file instance then send EmailMessage instance
        :param message_content: Mail content
        :param message_setting_dict: Dict include SUBJECT FROM TO and another EmailMessage Key and Value
        :param attach_file: File path as str
        :param use_html: Enable HTML format (If attach file is html)
        :return: None
        """
        mail_thunder_logger.info(
            f"smtp_create_message_with_attach_and_send, message_content: {message_content}, "
            f"message_setting_dict: {message_setting_dict}, attach_file:{attach_file}, use_html:{use_html}")
        try:
            self.send_message(
                self.smtp_create_message_with_attach(message_content, message_setting_dict, attach_file, use_html))
        except Exception as error:
            mail_thunder_logger.info(
                f"smtp_create_message_with_attach_and_send, message_content: {message_content}, "
                f"message_setting_dict: {message_setting_dict}, attach_file:{attach_file}, "
                f"use_html:{use_html}, failed: {repr(error)}")

    def smtp_create_message_and_send(self, message_content: str, message_setting_dict: dict, **kwargs):
        """
        Create new EmailMessage instance then send EmailMessage instance
        :param message_content: Mail content
        :param message_setting_dict: Dict include SUBJECT FROM TO and another EmailMessage Key and Value
        :return: None
        """
        mail_thunder_logger.info(
            f"smtp_create_message_and_send, message_content: {message_content}, "
            f"message_setting_dict: {message_setting_dict}, params:{kwargs}")
        try:
            self.send_message(self.smtp_create_message(message_content, message_setting_dict, **kwargs))
        except Exception as error:
            mail_thunder_logger.info(
                f"smtp_create_message_and_send, message_content: {message_content}, "
                f"message_setting_dict: {message_setting_dict}, params:{kwargs}, failed: {repr(error)}")


smtp_instance = SMTPWrapper()



class MailThunderException(Exception):
    pass


class MailThunderJsonException(MailThunderException):
    pass


class MailThunderContentException(MailThunderException):
    pass


class MailThunderArgparseException(MailThunderException):
    pass


class ExecuteActionException(MailThunderException):
    pass


class AddCommandException(MailThunderException):
    pass


class JsonActionException(MailThunderException):
    pass



import json
import os
from pathlib import Path
from threading import Lock

from je_mail_thunder.utils.exception.exceptions import MailThunderContentException
from je_mail_thunder.utils.json_format.json_process import reformat_json
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_data import mail_thunder_content_data_dict


def read_output_content():
    """
    read the editor content
    """
    lock = Lock()
    try:
        lock.acquire()
        cwd = str(Path.cwd())
        file_path = Path(cwd + "/mail_thunder_content.json")
        if file_path.exists() and file_path.is_file():
            with open(cwd + "/mail_thunder_content.json", "r+") as read_file:
                user_info = json.loads(read_file.read())
                mail_thunder_content_data_dict.update(user_info)
                return user_info
    except MailThunderContentException:
        raise MailThunderContentException
    finally:
        lock.release()


def write_output_content():
    """
    write the editor content
    """
    lock = Lock()
    try:
        lock.acquire()
        cwd = str(Path.cwd())
        with open(cwd + "/mail_thunder_content.json", "w+") as file_to_write:
            file_to_write.write(reformat_json(json.dumps(mail_thunder_content_data_dict)))
    except MailThunderContentException:
        raise MailThunderContentException
    finally:
        lock.release()



# SMTP Wrapper
from je_mail_thunder.imap.imap_wrapper import IMAPWrapper, imap_instance
# IMAP Wrapper
from je_mail_thunder.smtp.smtp_wrapper import SMTPWrapper, smtp_instance
# Content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_data import is_need_to_save_content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_data import mail_thunder_content_data_dict
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import read_output_content
from je_mail_thunder.utils.save_mail_user_content.mail_thunder_content_save import write_output_content
from je_mail_thunder.utils.save_mail_user_content.save_on_env import get_mail_thunder_os_environ
# Env
from je_mail_thunder.utils.save_mail_user_content.save_on_env import set_mail_thunder_os_environ
# JSON
from je_mail_thunder.utils.json.json_file import read_action_json
# File
from je_mail_thunder.utils.file_process.get_dir_file_list import get_dir_files_as_list
# Execute
from je_mail_thunder.utils.executor.action_executor import execute_action, execute_files, add_command_to_executor
# Project
from je_mail_thunder.utils.project.create_project_structure import create_project_dir
__all__ = [
    "IMAPWrapper", "imap_instance", "SMTPWrapper", "smtp_instance", "is_need_to_save_content",
    "mail_thunder_content_data_dict", "read_output_content", "write_output_content", "get_mail_thunder_os_environ",
    "set_mail_thunder_os_environ", "read_action_json", "get_dir_files_as_list", "execute_action", "execute_files",
    "add_command_to_executor", "create_project_dir"
]



from os import environ


def set_mail_thunder_os_environ(mail_thunder_user: str, mail_thunder_user_password: str):
    environ.update({"mail_thunder_user": mail_thunder_user})
    environ.update({"mail_thunder_user_password": mail_thunder_user_password})


def get_mail_thunder_os_environ():
    return {
        "mail_thunder_user": environ.get("mail_thunder_user", None),
        "mail_thunder_user_password": environ.get("mail_thunder_user_password", None)
    }





mail_thunder_content_data_dict = {
    "user": None,
    "password": None
}


def is_need_to_save_content():
    for value in mail_thunder_content_data_dict.values():
        if value is not None:
            return True
        else:
            return False
